'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _spawnCommand = require('spawn-command');

var _spawnCommand2 = _interopRequireDefault(_spawnCommand);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _safe = require('colors/safe');

var _safe2 = _interopRequireDefault(_safe);

var _lodash = require('lodash');

var _findUp = require('find-up');

var _managePath = require('manage-path');

var _managePath2 = _interopRequireDefault(_managePath);

var _arrify = require('arrify');

var _arrify2 = _interopRequireDefault(_arrify);

var _getScriptToRun = require('./get-script-to-run');

var _getScriptToRun2 = _interopRequireDefault(_getScriptToRun);

var _getScriptsFromConfig = require('./get-scripts-from-config');

var _getScriptsFromConfig2 = _interopRequireDefault(_getScriptsFromConfig);

var _getLogger = require('./get-logger');

var _getLogger2 = _interopRequireDefault(_getLogger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var noop = function noop() {}; // eslint-disable-line func-style, no-empty-function

exports.default = runPackageScripts;


function runPackageScripts(_ref) {
  var scriptConfig = _ref.scriptConfig;
  var scripts = _ref.scripts;
  var args = _ref.args;
  var _ref$options = _ref.options;
  var options = _ref$options === undefined ? {} : _ref$options;
  var callback = arguments.length <= 1 || arguments[1] === undefined ? noop : arguments[1];

  if (scripts.length === 0) {
    scripts = ['default'];
  }
  var scriptNames = (0, _arrify2.default)(scripts);
  var method = options.parallel ? 'map' : 'mapSeries';
  _async2.default[method](scriptNames, function (scriptName, cb) {
    var child = runPackageScript({ scriptConfig: scriptConfig, options: options, scriptName: scriptName, args: args });
    if (child.on) {
      child.on('exit', function (exitCode) {
        return cb(null, exitCode);
      });
    } else {
      cb(child);
    }
  }, function (err, results) {
    if (err) {
      callback({ error: err });
    } else {
      (function () {
        var NON_ERROR = 0;
        var result = (0, _lodash.find)(results, function (r) {
          return r !== NON_ERROR;
        });
        callback({ code: result });
      })();
    }
  });
}

function runPackageScript(_ref2) {
  var scriptConfig = _ref2.scriptConfig;
  var options = _ref2.options;
  var scriptName = _ref2.scriptName;
  var args = _ref2.args;

  var scripts = (0, _getScriptsFromConfig2.default)(scriptConfig, scriptName);
  var script = (0, _getScriptToRun2.default)(scripts, scriptName);
  if (!(0, _lodash.isString)(script)) {
    return {
      message: _safe2.default.red('Scripts must resolve to strings. There is no script that can be resolved from "' + scriptName + '"'),
      ref: 'missing-script'
    };
  }
  var command = [script, args].join(' ').trim();
  var log = (0, _getLogger2.default)(getLogLevel(options));
  log.info(_safe2.default.gray('p-s executing: ') + _safe2.default.green(command));
  return (0, _spawnCommand2.default)(command, { stdio: 'inherit', env: getEnv() });
}

function getLogLevel(_ref3) {
  var silent = _ref3.silent;
  var logLevel = _ref3.logLevel;

  if (logLevel) {
    return logLevel;
  } else if (silent) {
    return 'disable';
  } else {
    return undefined;
  }
}

function getEnv() {
  var env = (0, _lodash.clone)(process.env);
  var alterPath = (0, _managePath2.default)(env);
  var npmBin = (0, _findUp.sync)('node_modules/.bin');
  if (npmBin) {
    alterPath.unshift(npmBin);
  }
  return env;
}